# Generated by roxygen2: do not edit by hand

export(.atomicMasses)
export(.checkKnitrProt)
export(.checkSetupGroups)
export(.commonSpecies)
export(.extrSpecPref)
export(.imputeNA)
export(.parseFastaHeader)
export(.plotQuantDistr)
export(AAmass)
export(AucROC)
export(VolcanoPlotW2)
export(cleanListCoNames)
export(combineMultFilterNAimput)
export(convAASeq2mass)
export(corColumnOrder)
export(countNoOfCommonPeptides)
export(exportAsWombatP)
export(exportSdrfDraft)
export(extrSpeciesAnnot)
export(extractTestingResults)
export(foldChangeArrow2)
export(fuseProteomicsProjects)
export(getUPS1acc)
export(inspectSpeciesIndic)
export(isolNAneighb)
export(massDeFormula)
export(matrixNAinspect)
export(matrixNAneighbourImpute)
export(plotROC)
export(razorNoFilter)
export(readAlphaPeptFile)
export(readDiaNNFile)
export(readDiaNNPeptides)
export(readFasta2)
export(readFragpipeFile)
export(readIonbotPeptides)
export(readMassChroQFile)
export(readMaxQuantFile)
export(readMaxQuantPeptides)
export(readOpenMSFile)
export(readProlineFile)
export(readProtDiscovFile)
export(readProtDiscovererPeptides)
export(readProteomeDiscovererFile)
export(readProteomeDiscovererPeptides)
export(readSampleMetaData)
export(readSdrf)
export(readUCSCtable)
export(readUniProtExport)
export(readWombatNormFile)
export(removeSampleInList)
export(replMissingProtNames)
export(shortSoftwName)
export(summarizeForROC)
export(test2grp)
export(testRobustToNAimputation)
export(writeFasta2)
